<?php

/**
 * This file is part of the eZ Publish Kernel package.
 *
 * @copyright Copyright (C) eZ Systems AS. All rights reserved.
 * @license For full copyright and license information view LICENSE file distributed with this source code.
 */
namespace eZ\Publish\Core\MVC\Symfony\Cache\Http\EventListener;

use eZ\Publish\API\Repository\LocationService;
use eZ\Publish\Core\MVC\Symfony\Event\ContentCacheClearEvent;
use eZ\Publish\Core\MVC\Symfony\MVCEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Adds parent locations to the Http cache clear list, for given content.
 * Will consider direct parent for all content's locations.
 *
 * @deprecated since 6.8. Smart cache is replaced by http-cache multi-tagging from the platform-http-cache package.
 */
class ParentLocationsListener implements EventSubscriberInterface
{
    /**
     * @var LocationService
     */
    private $locationService;

    public function __construct(LocationService $locationService)
    {
        $this->locationService = $locationService;
    }

    public static function getSubscribedEvents()
    {
        return [MVCEvents::CACHE_CLEAR_CONTENT => ['onContentCacheClear', 100]];
    }

    public function onContentCacheClear(ContentCacheClearEvent $event)
    {
        $contentInfo = $event->getContentInfo();
        foreach ($this->locationService->loadLocations($contentInfo) as $location) {
            $event->addLocationToClear($this->locationService->loadLocation($location->parentLocationId));
        }
    }
}
